import json
import logging
import math
import time
import sacrebleu
import ctranslate2
import transformers
import sys
import os

log_file_path = 'work/logs/MtModelTraning/evaluation.log'
logging.basicConfig(filename=log_file_path, format='%(asctime)s - %(levelname)s - %(message)s', level=logging.INFO,
                    filemode='a')
logger = logging.getLogger()


def jakozh_length(text):
    if text is None:
        return 0
    lenText = len(text)
    lenText_utf8 = len(text.encode('utf-8'))
    # utf-8一个汉字占3个字符，减去原计数就是多出来的2/3，再除以2就是增量。再加回去即可
    size = int((lenText_utf8) / 3)
    return size


def get_data(data_path, size, max_seq_len, L_src, L_obj):
    datalist = []
    with open(data_path, encoding='utf-8') as file:
        lines = file.readlines()
        datalist = [(json.loads(line)['translation'][L_src], json.loads(line)['translation'][L_obj]) for line in lines]
    datalist = datalist[:min(size, len(datalist))]
    src_list = [line[0][:min(max_seq_len, len(line[0]))] for line in datalist]
    tgt_list = [[line[1][:min(max_seq_len, len(line[1]))] for line in datalist]]
    return src_list, tgt_list


def get_model(model_path, device_type, device_num):
    translator = ctranslate2.Translator(model_path, device_type, device_index=device_num)
    tokenizer = transformers.AutoTokenizer.from_pretrained(model_path)
    return translator, tokenizer


def model_infer(translator, tokenizer, valid_data, batch_size=8, L_obj="zh"):
    start_ts_time = time.time()
    sentences = [tokenizer.convert_ids_to_tokens(tokenizer.encode(seq)) for seq in valid_data]
    target_prefix = [tokenizer.lang_code_to_token[L_obj]]
    # results = translator.translate_batch(sentences, target_prefix=[target_prefix] * len(valid_data),
    #                                      max_batch_size=batch_size,disable_unk=True)
    results = translator.translate_batch(sentences, target_prefix=[target_prefix] * len(valid_data),
                                         max_batch_size=batch_size)
    targets = [result.hypotheses[0][1:] for result in results]
    infer_results = [tokenizer.decode(tokenizer.convert_tokens_to_ids(target)) for target in targets]
    end_ts_time = time.time()

    # print(valid_data)
    # print(infer_results)

    return infer_results, end_ts_time - start_ts_time


def evaluation(data_path, translator, tokenizer, size, l_src, l_obj, batch_size=8):
    if l_src == "ja" or l_src == "zh":
        max_seq_len = 256
    else:
        max_seq_len = 320

    # init test data
    valid_data_src, valid_data_obj = get_data(data_path, size, max_seq_len, l_src, l_obj)
    # print(valid_data_src)
    # valid data infer
    tokenizer.src_lang = l_src
    infer_results, infer_time = model_infer(translator, tokenizer, valid_data_src, batch_size, l_obj)

    # print(valid_data_src)
    # print(infer_results)
    # print(valid_data_obj[0])

    # for i in range(len(valid_data_src)):
    #     with open('/disk1/work/liaozm/Translate_'+L_src+'_result_evaluation/Test_'+l_src+'.txt', mode='a', encoding='utf-8') as filename:
    #         filename.write(valid_data_src[i])
    #         filename.write('\n')  # 换行
    #         filename.write("Xiaoyi:"+infer_results[i])
    #         filename.write('\n')  # 换行
    #         filename.write("Google:"+valid_data_obj[0][i])
    #         filename.write('\n')  # 换行
    #
    # for i in range(len(infer_results)):
    #     with open('/disk1/work/liaozm/Test_Lt_inferresult.txt', mode='a', encoding='utf-8') as filename:
    #         filename.write(infer_results[i])
    #         filename.write('\n')  # 换行
    # for i in range(len(valid_data_obj[0])):
    #     with open('/disk1/work/liaozm/uk_Googleresult.txt', mode='a', encoding='utf-8') as filename:
    #         filename.write(valid_data_obj[0][i])
    #         filename.write('\n')  # 换行

    # print("Model_inference:",infer_results)
    # print("Google_inference:",valid_data_obj)

    bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    print(f"The_Bleu({l_src})={bleu.score}")

    token_count = sum(map(lambda x: len(tokenizer(x, return_tensors="pt")["input_ids"][0]), valid_data_src))
    if l_src == "ko" or l_src == "ja" or l_src == "zh":
        word_count = sum(map(lambda x: jakozh_length(x), valid_data_src))
    else:
        word_count = sum(map(lambda x: len(x.split(" ")), valid_data_src))
    logger.info(
        f"""======================CT2 infer======================
            Bleu  Info: bleu_score:{bleu.score},lang:{l_src},batch_size:{batch_size},data_path:{data_path}
            Speed Info: word_count:{word_count},word_speed:{word_count / infer_time} words/second,token_count:{token_count},token_speed:{token_count / infer_time} tokens/second 
            """
    )
    # print(f"""======================CT2 infer======================
    #         Bleu  Info: bleu_score:{bleu.score},lang:{l_src},batch_size:{batch_size},data_path:{data_path}
    #         Speed Info: word_count:{word_count},word_speed:{word_count / infer_time} words/second,token_count:{token_count},token_speed:{token_count / infer_time} tokens/second
    #         """)


def en_eval(model_path, device_num, L_src, L_obj):
    valid_data_map = {
        "en": "/disk1/work/MtCorpus/M2M100_Corpus/En_Data/News_Valid.json",
        # "en": "/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Place.json",
        # "en": "/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Name.json",
        # "en":"/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Valid.json",
        # "en": "/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Institution/en.json",
        # "en": "/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Institution/x.json",
    }
    size = 10000
    batch_size = 1280
    # init model
    translator, tokenizer = get_model(model_path, "cuda", device_num)
    # evaluation
    for key in valid_data_map.keys():
        evaluation(data_path=valid_data_map[key], translator=translator, tokenizer=tokenizer,
                   l_src=key, l_obj="zh", size=size, batch_size=batch_size)


def latn_eval(model_path, device_num, L_src, L_obj):
    valid_data_map = {
        "bg": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/bg_valid.json",
        # "vi": "/disk1/work/liaozm/Multi_Latn_Training/Valid_data/vi_upper.json",
        "fa": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/fa_valid.json",
        "sk": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/sk_valid.json",
        "et": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/et_valid.json",
        "vi": "/disk1/work/liaozm/Multi_Latn_Training/Valid_data/vi.json",
        "hu": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/hu_valid.json",
        "uk": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/uk_valid.json",
        "tr": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/tr_valid.json",
        "lt": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/lt_valid.json",
        "ur": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ur_valid.json",
        "it": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/it_valid.json",
        "ne": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ne_valid.json",
        "hi": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/hi_valid.json",
        "ps": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ps_valid.json",
        "sq": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/sq_valid.json",
        "ar": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ar_valid.json",
        "de": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/de_valid.json",
        "ru": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ru_valid.json",
        "ro": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ro_valid.json",
        "sr": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/sr_valid.json",
        "nl": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/nl_valid.json",
        "cs": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/cs_valid.json",
        "el": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/el_valid.json",
        "pl": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/pl_valid.json",
        "fr": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/fr_valid.json",
        "es": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/es_valid.json",
        "pt": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/pt_valid.json",

    }
    size = 10000
    batch_size = 1280
    # init model
    translator, tokenizer = get_model(model_path, "cuda", device_num)

    # evaluation
    for key in valid_data_map.keys():
        evaluation(data_path=valid_data_map[key], translator=translator, tokenizer=tokenizer,
                   l_src=key, l_obj="zh", size=size, batch_size=batch_size)


def zh_eval(model_path, device_num, L_src, L_obj):
    valid_data_map = {
        "zh":"/disk1/work/MtCorpus/M2M100_Corpus/En_Data/News_Valid.json"
    }
    size = 10000
    batch_size = 1280
    # init model
    translator, tokenizer = get_model(model_path, "cuda", device_num)
    # evaluation
    for key in valid_data_map.keys():
        evaluation(data_path=valid_data_map[key], translator=translator, tokenizer=tokenizer,
                   l_src=key, l_obj=L_obj, size=size, batch_size=batch_size)


def jako_eval(model_path, device_num, L_src, L_obj):
    valid_data_map = {
        "mn": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/mn_valid.json",
        "ja": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ja_valid.json",
        "ko": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/ko_valid.json",
        "km": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/km_valid.json",
        "th": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/th_valid.json",
        "lo": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/lo_valid.json",
        "id": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/id_valid.json",
        "my": "/disk1/work/MtCorpus/M2M100_Corpus/Valid_data/my_valid.json",
    }
    size = 10000
    batch_size = 1280

    # init model
    translator, tokenizer = get_model(model_path, "cuda", device_num)
    # evaluation
    for key in valid_data_map.keys():
        evaluation(data_path=valid_data_map[key], translator=translator, tokenizer=tokenizer,
                   l_src=key, l_obj="zh", size=size, batch_size=batch_size)


if __name__ == "__main__":
    L_src="latn"
    # L_src = "jako"
    # L_src = "en"
    # L_src="zh"
    device_num = 7
    if (L_src == "en"):
        L_obj = "zh"
        # model_path="/disk1/work/MtModel/M2M100/enzh_model/fp16_M2M_20240122_en_special"
        model_path="/disk1/work/liaozm/Multi_enzh_Training/fp16_M2M_20240204_enzh_special"
        # model_path="/disk1/work/liaozm/Multi_enzh_Training/C2"
        # model_path = "/disk1/work/MtModel/M2M100/enzh_model/fp16_M2M_20240204_enzh_special_institution"
        model_path="/disk1/work/liaozm/Multi_enzh_Training/C2_2024_0218_en_name_place"
        model_path = "/disk1/work/liaozm/Multi_enzh_Training/C2_2024_0218_en_name_place_institution"
        en_eval(model_path, device_num, L_src, L_obj)
    elif (L_src == "jako"):
        L_obj = "zh"
        model_path="/disk1/work/MtModel/M2M100/jako_model/fp16_M2M_20231206_km_th"
        jako_eval(model_path, device_num, L_src, L_obj)
    elif (L_src == "latn"):
        L_obj = "zh"
        model_path="/disk1/work/MtModel/M2M100/latn_model/fp16_M2M_20231205_fa_et_sk"
        model_path="/disk1/work/liaozm/Multi_Latn_Training/fp16_result_model_20240327"
        # model_path="/disk1/work/MtModel/M2M100/latn_model/20240325_latn_model_bg"
        latn_eval(model_path, device_num, L_src, L_obj)
    elif (L_src == "zh"):
        L_obj = "en"
        model_path="/disk1/work/MtModel/M2M100/enzh_model/fp16_M2M_20240131_enzh"
        model_path="/disk1/work/liaozm/Multi_enzh_Training/fp16_M2M_20240204_enzh_special"
        zh_eval(model_path, device_num, "zh", 'en')
        model_path = "/disk1/work/liaozm/Multi_enzh_Training/C2"
    else:
        print("Your input exist error")

