import json
import sacrebleu
from transformers import M2M100ForConditionalGeneration, M2M100Tokenizer, pipeline
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, pipeline
import torch
import warnings
import sys
warnings.filterwarnings("ignore")

def get_data(data_path, size, max_seq_len, L_src, L_obj):
    datalist = []
    with open(data_path, encoding='utf-8') as file:
        lines = file.readlines()
        datalist = [(json.loads(line)['translation'][L_src], json.loads(line)['translation'][L_obj]) for line in lines]
    datalist = datalist[:min(size, len(datalist))]
    src_list = [line[0][:min(max_seq_len, len(line[0]))] for line in datalist]
    tgt_list = [[line[1][:min(max_seq_len, len(line[1]))] for line in datalist]]
    return src_list, tgt_list



def model_infer(model,model_path,L_src,device,valid_data, batch_size,obj="zh"):
    tokenizer = M2M100Tokenizer.from_pretrained(model_path, src_lang=L_src, tgt_lang=obj)

    # tokenizer = AutoTokenizer.from_pretrained(model_path,src_lang=L_src,tgt_lang=obj)
    translator = pipeline('translation', model=model, tokenizer=tokenizer, src_lang=L_src, tgt_lang=obj, device=device)
    result=[]
    for con in valid_data:
        result.append(translator(con,batch_size=batch_size)[0]["translation_text"])
    return result

def en_eval(model_path,device_num):
    size = 10
    batch_size = 64
    max_seq_len=320
    data_path="valid_data/en_valid.json"
    # init model
    device = torch.device("cuda:" + str(device_num))
    # model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    model = AutoModelForSeq2SeqLM.from_pretrained(model_path).to(device)

    valid_data_src, valid_data_obj = get_data(data_path, size, max_seq_len, "en", "zh")
    print(valid_data_obj)
    infer_results=model_infer(model,model_path,"en",device,valid_data_src,batch_size)
    print(infer_results)
    bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    print("en:",bleu.score)

def zh_eval(model_path,device_num):
    size = 10
    batch_size = 256
    max_seq_len=256
    data_path="/disk1/work/MtCorpus/Total_English_Data/News_Valid.json"
    data_path= "/disk1/work/MtCorpus/En_Data/News_Valid.json"
    # init model
    device = torch.device("cuda:" + str(device_num))
    model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    # model = AutoModelForSeq2SeqLM.from_pretrained(model_path).to(device)

    valid_data_src, valid_data_obj = get_data(data_path, size, max_seq_len, "zh", "en")

    print(valid_data_src)
    infer_results=model_infer(model,model_path,"zh",device,valid_data_src,batch_size)
    print(infer_results)
    bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    print("en:",bleu.score)

def jako_eval(model_path,device_num):
    size = 1000
    batch_size = 16
    max_seq_len=320
    valid_data_map = {
        "ja": "valid_data/ja_valid.json",
        "ko": "valid_data/ko_valid.json",
    }
    # init model
    device = torch.device("cuda:" + str(device_num))
    model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    # model = AutoModelForSeq2SeqLM.from_pretrained(model_path).to(device)
    for key in valid_data_map.keys():
        valid_data_src, valid_data_obj = get_data(valid_data_map[key], size, max_seq_len, key, "zh")
        infer_results=model_infer(model,model_path,key,device,valid_data_src,batch_size)
        bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
        print(key+":",bleu.score)

def latn_eval(model_path,device_num):
    size = 1000
    batch_size = 16
    max_seq_len=320
    valid_data_map = {
        "de": "valid_data/de_valid.json",
        "ru": "valid_data/ru_valid.json",
        "ro": "valid_data/ro_valid.json",
        "sr": "valid_data/sr_valid.json",
        "nl": "valid_data/nl_valid.json",
        "cs": "valid_data/cs_valid.json",
        "el": "valid_data/el_valid.json",
        "pl": "valid_data/pl_valid.json",
        "fr": "valid_data/fr_valid.json",
        "es": "valid_data/es_valid.json",
        "pt": "valid_data/pt_valid.json",
    }
    # init model
    device = torch.device("cuda:" + str(device_num))
    model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    for key in valid_data_map.keys():
        valid_data_src, valid_data_obj = get_data(valid_data_map[key], size, max_seq_len, key, "zh")
        infer_results=model_infer(model,model_path,key,device,valid_data_src,batch_size)
        bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
        print(key+":",bleu.score)

def ar_eval(model_path,device_num):
    # size = 10
    batch_size = 16
    max_seq_len=320
    # data_path="/disk1/work/MtCorpus/Total_English_Data/News_Valid.json"
    # init model
    device = torch.device("cuda:" + str(device_num))
    model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    valid_data_src=["كما وجهت باستحداث جائزة كبرى للمسابقة باسم المطرب الراحل الفنان محسن فاروق، عرفاناً بدوره في تأسيس المسابقة، وإلى كونه أحد فرسان الدفاع عن هوية مصر الموسيقية وتراثها، ومن الجيل الأول لفرقة الموسيقي العربية بدار الأوبرا المصرية."]
    valid_data_obj=[["它还指导以已故歌手 Mohsen Farouk 的名义为比赛设立大奖，以感谢他在建立比赛中的作用，以及作为捍卫埃及音乐身份和遗产的骑士之一，从一开始埃及歌剧院阿拉伯音乐团体的一代。"]]
    infer_results=model_infer(model,model_path,"ar",device,valid_data_src,batch_size)
    print(infer_results)
    print(valid_data_obj[0])
    bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    print("ar:",bleu.score)

def en_bak_eval(model_path,device_num):
    # size = 10
    batch_size = 16
    max_seq_len=320
    # data_path="/disk1/work/MtCorpus/Total_English_Data/News_Valid.json"
    # init model
    device = torch.device("cuda:" + str(device_num))
    model = M2M100ForConditionalGeneration.from_pretrained(model_path).to(device)
    # valid_data_src=["كما وجهت باستحداث جائزة كبرى للمسابقة باسم المطرب الراحل الفنان محسن فاروق، عرفاناً بدوره في تأسيس المسابقة، وإلى كونه أحد فرسان الدفاع عن هوية مصر الموسيقية وتراثها، ومن الجيل الأول لفرقة الموسيقي العربية بدار الأوبرا المصرية."]
    # valid_data_obj=[["它还指导以已故歌手 Mohsen Farouk 的名义为比赛设立大奖，以感谢他在建立比赛中的作用，以及作为捍卫埃及音乐身份和遗产的骑士之一，从一开始埃及歌剧院阿拉伯音乐团体的一代。"]]

    valid_data_src=["Yi Zhao"]
    infer_results=model_infer(model,model_path,"en",device,valid_data_src,batch_size)
    print(infer_results)
    # print(valid_data_obj[0])
    # bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    # print("ar:",bleu.score)


if __name__ == "__main__":
    # L_src=str(sys.argv[1])
    # device_num=int(sys.argv[2])
    L_src = "en"
    device_num = 7
    # try:
    if(L_src=="en"):
        # model_path = "/disk1/work/MtModel/New_English_Model3"
        # model_path = "/disk1/work/liaozm/NLLB"
        model_path="/disk1/work/liaozm/Result_model2"
        # model_path="/disk1/work/liaozm/Exist_Trans_Model/M2M100/M2M100_1.2B"
        en_bak_eval(model_path,device_num)
    elif(L_src=="jako"):
        model_path = "/disk1/work/MtModel/KO_JA_Model_v2"
        jako_eval(model_path,device_num)
    elif(L_src=="latn"):
        model_path = "/disk1/work/MtModel/Latn_Model_v4"

        latn_eval(model_path,device_num)
    elif (L_src == "zh"):
        L_obj = "en"
        model_path = "facebook/m2m100_418M"
        model_path ="/disk1/work/liaozm/Exist_Trans_Model/Orignal_Model/en_model"
        model_path = "/disk1/work/liaozm/NLLB"
        zh_eval(model_path, device_num)
    elif(L_src=="ar"):
        L_obj = "ar"
        model_path = "facebook/m2m100_418M"
        # model_path = "/disk1/work/MtModel/KO_JA_Model_v2"
        ar_eval(model_path, device_num)
    else:
        print("Your input exist error")
    # except:
    #     print("Your input exist error")













