import logging
import ctranslate2
import transformers
import re
import time
import emoji
import requests
from typing import List
import json
from json import JSONEncoder


def get_model(model_path, device_type, device_num,L_src):
    # translator = ctranslate2.Translator(model_path, device_type, device_index=device_num)
    translator = ctranslate2.Translator(model_path, "cuda")
    tokenizer = transformers.AutoTokenizer.from_pretrained(model_path,src_lang=L_src)
    return translator, tokenizer

def model_infer(model_path, device_num, Sentence, batch_size=64,L_src="en"):
    start_ts_time = time.time()
    # translator, tokenizer = get_model(model_path, "cuda", device_num)
    translator, tokenizer = get_model(model_path, "cuda",device_num,L_src)
    # data = []
    # data.append(Sentence)

    data=Sentence

    sentences = [tokenizer.convert_ids_to_tokens(tokenizer.encode(seq)) for seq in data]
    target_prefix = [tokenizer.lang_code_to_token["zh"]]
    # target_prefix=["zho_Hans"]
    # results = translator.translate_batch(sentences, target_prefix=[target_prefix] * len(data),max_batch_size=batch_size,repetition_penalty=2,disable_unk=True,replace_unknowns=True)
    results = translator.translate_batch(sentences, target_prefix=[target_prefix] * len(data),max_batch_size=batch_size,disable_unk=True,repetition_penalty=2)
    targets = [result.hypotheses[0][1:] for result in results]
    infer_results = [tokenizer.decode(tokenizer.convert_tokens_to_ids(target)) for target in targets]
    end_ts_time = time.time()
    # word_count = sum(map(lambda x: len(x.split(" ")), valid_data_src))
    # print("word_count=",word_count,"speed=",(word_count / (end_ts_time - start_ts_time+12)))
    return infer_results, end_ts_time - start_ts_time

#对美国来说，北约转向东亚具有重要的战略意义。美国政府正寻求将其松散的联盟网络整合成一个更牢固的、能遏制中国崛起的联盟，从而维持其全球霸权地位。

if __name__ == "__main__":
    special_word_map = {
        "USFS": "美国国家森林局",
        "EOP": "美国总统行政办公室",
        "EPA": "美国国家环境保护局",
        "ASA": "美国统计协会",
        "NSA": "美国国家安全局",
        "NRO": "美国国家侦察局",
        "USSS": "美国特勤局",
        "USDA": "美国农业部",
        "FDA": "美国食品药品监督管理局",
        "NASA": "美国国家航空航天局",
        "DHS": "国土安全部",
        "IRS": "美国国内税收局",
        "FEMA": "联邦紧急管理署",
        "SEC": "美国证券交易委员会",
        "DEA": "美国缉毒局",
        "ATF": "美国酒精、烟草、火器与爆炸物管理局",
        "NTSB": "国家运输安全委员会",
        "NRC": "美国核能管理委员会",
        "CDC": "美国疾病控制与预防中心",
        "NIH": "国家卫生研究院",
        "TSA": "美国交通安全管理局",
        "USPS": "美国邮政服务",
        "HUD": "美国住房和城市发展部",
        "DOE": "美国能源部",
        "HHS": "美国卫生与人类服务部",
        "OSHA": "职业安全与健康管理局",
        "FCC": "美国联邦通信委员会",
        "ICE": "移民与海关执法局",
        "SSA": "美国社会保障局",
        "SBA": "小型企业管理局",
        "NLRB": "国家劳动关系委员会",
        "FEC": "联邦选举委员会",
        "NIST": "国家标准与技术研究院",
        "NARA": "国家档案和记录管理局",
        "CFTC": "商品期货交易委员会",
        "FAA": "美国联邦航空管理局",
        "VA": "美国退伍军人事务部",
        "EEOC": "平等雇佣机会委员会",
        "OPM": "美国办公室人事管理",
        "FDIC": "美国联邦存款保险公司",
        "NGA": "国家地理空间情报局",
        "DOT": "美国交通部",
        "NOAA": "美国国家海洋和大气管理局",
        "NEA": "美国国家艺术基金会",
        "NPS": "国家公园管理局",
        "NSF": "国家科学基金会",
        "OCC": "美国货币监理署",
        "PTO": "美国专利商标局",
        "TVA": "田纳西河谷管理局",
        "USTR": "美国贸易代表办公室",
        "CFPB": "美国消费者金融保护局",
        "CMS": "美国医疗保险和医疗服务中心",
        "FHFA": "联邦住房金融署",
        "FMCSA": "联邦汽车运输管理局",
        "FRA": "美国联邦铁路管理局",
        "FTC": "美国联邦贸易委员会",
        "GSA": "美国总务管理局",
        "NTIA": "美国国家电信和信息管理局",
        "OMB": "美国管理与预算办公室",
        "OPIC": "海外私人投资公司",
        "OSC": "美国特别顾问办公室",
        "OSTP": "美国科学与技术政策办公室",
        "PBGC": "美国职业年金保障公司",
        "AID": "美国国际开发署",
        "BEA": "美国经济分析局",
        "BIA": "美国印第安事务局",
        "BLS": "美国劳工统计局",
        "BOEM": "美国海洋能源管理局",
        "BOP": "联邦监狱局",
        "BPA": "美国西北电力管理局",
        "CBO": "国会预算办公室",
        "CEA": "白宫经济顾问委员会",
        "CPSC": "美国消费品安全委员会",
        "EIA": "美国能源信息署",
        "FAS": "美国农业外交局",
        "FCA": "联邦农业抵押贷款公司",
        "FINRA": "金融业监管局",
        "FMC": "联邦海事委员会",
        "FRB": "美国联邦储备系统",
        "FTA": "联邦交通管理局",
        "GAO": "美国政府问责办公室",
        "IHS": "印第安卫生服务局",
        "MSHA": "美国劳工部矿业安全与卫生管理局",
        "NCUA": "国家信用联盟管理局",
        "OIG": "联邦机构内部监察办公室",
        "ONDCP": "白宫国家毒品控制政策办公室",
        "MIT": "麻省理工学院",
        "Harvard": "哈佛大学",
        "Stanford": "斯坦福大学",
        "Yale": "耶鲁大学",
        "Princeton": "普林斯顿大学",
        "Columbia": "哥伦比亚大学",
        "Chicago": "芝加哥大学",
        "Berkeley": "加利福尼亚大学伯克利分校",
        "UPenn": "宾夕法尼亚大学",
        "Johns Hopkins": "约翰霍普金斯大学",
        "Northwestern": "西北大学",
        "Michigan": "密歇根大学",
        "Duke": "杜克大学",
        "Cornell": "康奈尔大学",
        "UCLA": "加利福尼亚大学洛杉矶分校",
        "Carnegie Mellon": "卡内基梅隆大学",
        "Brown": "布朗大学",
        "Virginia": "弗吉尼亚大学",
        "Georgetown": "乔治城大学",
        "Washington": "华盛顿大学",
        "Rice": "莱斯大学",
        "Emory": "埃默里大学",
        "UC San Diego": "加利福尼亚大学圣地亚哥分校",
        "Illinois": "伊利诺伊大学厄巴纳-香槟分校",
        "USC": "南加利福尼亚大学",
        "Tufts": "塔夫茨大学",
        "Vanderbilt": "范德堡大学",
        "Texas": "德克萨斯大学奥斯汀分校",
        "Dartmouth": "达特茅斯学院",
        "Maryland": "马里兰大学",
        "Case Western Reserve": "凯斯西储大学",
        "North Carolina": "北卡罗来纳大学教堂山分校",
        "Rochester": "罗切斯特大学",
        "Wisconsin": "威斯康辛大学麦迪逊分校",
        "Florida": "佛罗里达大学",
        "Illinois Chicago": "伊利诺伊芝加哥分校",
        "Minnesota": "明尼苏达大学",
        "UC Santa Barbara": "加利福尼亚大学圣巴巴拉分校",
        "Penn State": "宾夕法尼亚州立大学",
        "Purdue": "普渡大学",
        "Rutgers": "罗格斯大学",
        "Syracuse": "雪城大学",
        "Texas A&M": "德克萨斯农工大学",
        "Michigan State": "密歇根州立大学",
        "RBI": "印度储备银行",
        "ISRO": "印度空间研究组织",
        "UIDAI": "印度身份认证局",
        "CBI": "印度中央调查局",
        "DRDO": "印度国防研究与发展组织",
        "MEA": "印度外交部",
        "MHA": "印度内政部",
        "MoF": "印度财政部",
        "MoD": "印度国防部",
        "MoHRD": "印度人力资源发展部",
        "MoI&B": "印度信息与广播部",
        "MoSJE": "印度社会司法与权利保障部",
        "MoWCD": "印度妇女与儿童发展部",
        "NITI Aayog": "印度国家发展与改革委员会",
        "TRAI": "印度电信管理局",
        "SEBI": "印度证券交易委员会",
        "NPCI": "印度支付公司",
        "NHAI": "印度国家公路局",
        "NIC": "印度国家信息中心",
        "FCI": "印度粮食公司",
        "EPFO": "印度雇员提供的公积金组织",
        "ESIC": "印度员工国家保险公司",
        "ICMR": "印度医学研究理事会",
        "CSIR": "印度科学与工业研究委员会",
        "ICAR": "印度农业研究委员会",
        "AICTE": "印度工业培训委员会",
        "UGC": "印度大学教育委员会",
        "BARC": "印度原子能研究中心",
        "DAE": "原子能部",
        "ISID": "印度社会和经济发展研究所",
        "FSSAI": "印度食品安全标准局",
        "NABARD": "印度国家农村和农村发展银行",
        "IRDAI": "印度保险监督与发展局",
        "CBDT": "印度税收部",
        "CBIC": "印度关税与消费税委员会",
        "NPPA": "印度国家药品定价局",
        "NIDHI": "印度国家创业发展公司",
        "CCMB": "印度中心分子生物学",
        "NCERT": "印度国家教育研究与培训委员会",
        "NCPCR": "印度国家儿童权利保护委员会",
        "NCW": "印度国家妇女委员会",
        "NHRC": "印度国家人权委员会",
        "NHRM": "印度国家农村卫生任务",
        "NACO": "印度国家艾滋病控制组织",
        "NHB": "印度国家住房银行",
        "NIELIT": "印度国家电子与信息技术研究所",
        "NCRB": "印度国家犯罪记录局",
        "NCST": "印度国家计划部落委员会",
        "NMMC": "新孟买市政公司",
        "AIIMS": "全印度医学科学研究所",
        "IISc": "印度科学学院",
        "BHEL": "印度电力公司有限公司",
        "DGCA": "印度民用航空总局",
        "SEZ": "特殊经济区",
        "PWD": "公共工程部",
        "TIFR": "印度核研究所",
        "HAL": "印度斯坦航空有限公司",
        "NALCO": "印度国家铝公司",
        "NTPC": "印度国家热电公司",
        "BEL": "印度电子有限公司",
        "ITDC": "印度旅游发展公司",
        "CPWD": "印度中央公共工程部",
        "BIS": "印度标准局",
        "AAI": "印度机场管理局",
        "CWC": "印度中央仓储公司",
        "ONGC": "印度石油自然气公司",
        "PFRDA": "养老基金管理局",
        "ITBP": "印度藏区边防警察",
        "IIT": "印度理工学院",
        "DU": "德里大学",
        "JNU": "贾瓦哈拉尔尼赫鲁大学",
        "JU": "加尔各答大学",
        "BHU": "班纳拉斯印度大学",
        "AMU": "阿拉伯摩阿拉大学",
        "IGNOU": "印度国际开放大学",
        "NIT": "国立工程学院",
        "IIM": "印度管理学院",
        "BITS": "比尔拉理工科学院",
        "VIT": "瓦拉纳西理工科学院",
        "SRM": "斯里兰卡拉曼理工科学院",
        "JMI": "约旦伊斯兰教大学",
        "IISER": "印度科学教育与研究学院",
        "FTII": "印度电影与电视学院",
        "NIFT": "国家时尚技术学院",
        "PU": "旁遮普大学",
        "CU": "加尔各答大学",
        "DTU": "德里工业大学",
        "MDU": "穆鲁纳卡尔大学",
        "SRFTI": "国家电影与电视学院",
        "NLU": "国立法学院",
        "IIIT": "国际信息技术研究所",
        "MANUU": "马努大学",
        "IIST": "印度航天科学技术学院",
        "KIIT": "基特大学",
        "LPU": "拉维亚尼兰通用大学",
        "JGU": "哥吉拉特大学",
        "SRHU": "梅里特拉霍伊拉吉大学",
        "TERI": "能源和资源研究学院",
        "IIEST": "印度工程科学与技术学院",
        "NLUJ": "国家法学院贾德普尔",
        "MU": "孟加拉大学",
        "NLUO": "国家法学院奥里萨",
        "CUSAT": "科钦大学",
        "SPJIMR": "斯里帕德马拉杰曼格拉大学",
        "IGIDR": "印度经济增长与发展研究所",
        "EC": "欧洲委员会",
        "EP": "欧洲议会",
        "EUCO": "欧洲理事会",
        "ECA": "欧洲法院",
        "ECB": "欧洲中央银行",
        "EESC": "欧洲经济社会委员会",
        "CoR": "欧洲委员会",
        "EUO": "欧洲石油监察机构",
        "EIB": "欧洲投资银行",
        "EAS": "欧洲外交行动服务",
        "EASO": "欧洲难民支援办公室",
        "ELA": "欧洲劳动局",
        "EEA": "欧洲环境署",
        "EMCDDA": "欧洲毒品和药物滥用监测中心",
        "EIGE": "欧洲性别平等研究所",
        "EIF": "欧洲投资基金",
        "EMCC": "欧洲劳动力观察中心",
        "EIT": "欧洲创新与技术研究所",
        "EMSA": "欧洲海事安全局",
        "ENISA": "欧洲网络与信息安全署",
        "ERA": "欧洲铁路局",
        "ESMA": "欧洲证券与市场管理局",
        "EUIPO": "欧盟知识产权局",
        "Eurojust": "欧洲司法合作机构",
        "eu-LISA": "欧洲立法信息与透明度机构",
        "EUMC": "欧洲联合军事委员会",
        "EUPC": "欧洲政策中心",
        "EURES": "欧洲就业服务局",
        "Eurostat": "欧洲统计局",
        "EU-OSHA": "欧盟职业安全与健康署",
        "EBA": "欧洲银行业监管局",
        "ECHA": "欧洲化学品局",
        "ECON": "欧洲文化和教育委员会",
        "EEAS": "欧洲外交行动服务",
        "EEIG": "欧洲经济兴隆小组",
        "EFA": "欧洲金融管理局",
        "EFSA": "欧洲食品安全局",
        "EMA": "欧洲药品管理局",
        "EMN": "欧洲移民网络",
        "EPO": "欧洲专利局",
        "SESAR": "单一欧洲空中交通管理研究",
        "FRONTEX": "欧洲边境与海岸防卫局",
        "ETF": "欧洲培训基金",
        "Cedefop": "欧洲职业培训发展中心",
        "EAR": "欧洲银行业监管局",
        "EUROFOUND": "欧洲基金会",
        "EUREC": "欧洲可再生能源研究中心",
    }
    def specialword_replace(result):
        for key, value in special_word_map.items():
            result = replace_word(result, key, value)
        return result
    def replace_word(input_string, target_word, replacement):
        # 定义正则表达式模式，确保只匹配完整的单词
        pattern = re.compile(r'\b' + re.escape(target_word) + r'\b')
        # 进行替换
        result = re.sub(pattern, replacement, input_string)
        return result

    # Sentence="Three cases of facial nerve palsy were also reported in volunteers who received the Oxford/AstraZeneca vaccine during clinical trials, but there were also three cases in the placebo arm."

    Sentence=[]
    # x="IIT and IIIT"
    # print(specialword_replace(x))
    Sentence.append("Watch Congrates for 8 Crore People's of Tamil nadu on YouTube")
    # model_path="/disk1/work/liaozm/Multi_enzh_Training/C2_2024_0218_en_name_place"
    model_path="/disk1/work/liaozm/Multi_enzh_Training/C2_2024_0218_en_name_place_institution"
    L_src = "en"
    device_num = 6
    print(model_infer(model_path, device_num, Sentence, batch_size=128,L_src=L_src))


    # Sentence="hello world ⛪ ⛩ ⛲ ⛺ ♨ ⛽ ⚓ ⛵ ⛴ ✈ ⌛ ⏳ ⌚ ⏰ ⏱ ⏲ ☀ ⭐ ☁ ⛅ ⛈ ☂ ☔ ⛱ ⚡ ❄ ☃ ⛄"
    # print(Sentence)
    # print(emoji.replace_emoji(Sentence, ""))



    # Sentence = []

    # data_path="/disk1/work/MtCorpus/M2M100_Corpus/En_Data/News_Valid.json"
    # valid_data_src, valid_data_obj = get_data(data_path, size=10000, max_seq_len=256, L_src="en", L_obj="zh")
    #
    # L_src="en"
    # device_num = 6
    # result,time=model_infer(model_path, device_num, valid_data_src, batch_size=1280,L_src=L_src)
    # print(result)























    # class Message:
    #     def __init__(self,role,content):
    #         self.role=role
    #         self.content=content
    # class ChatPostRequestEncoder(JSONEncoder):
    #     def default(self,o):
    #         return o.__dict__
    # class ChatPostRequest:
    #     def __init__(self,model,messages,stream,max_tokens):
    #         self.model=model
    #         self.messages=messages
    #         self.stream=stream
    #         self.max_tokens=max_tokens
    #         self.temperature=0.01
    #         self.top_k=10
    #         self.top_p=0.95
    #
    # def post_llm_http_request(messages:List[Message],
    #                           api_url: str="http://192.168.8.158:8106/v1/chat/completions",
    #                           stream: bool =False,
    #                           max_tokens: int=512,
    #                           model="vicuna-7b-v1.5")-> requests.Response:
    #     headers={"Content-Type": "application/json"}
    #     chat_post_request=ChatPostRequest(model,messages,stream,max_tokens)
    #     post_json_str=json.dumps(chat_post_request,cls=ChatPostRequestEncoder)
    #     result_message=""
    #     try:
    #         response=requests.post(api_url,headers=headers,data=post_json_str)
    #         result_message=json.loads(response.text)['choices'][0]['message']['content']
    #     except Exception as e :
    #         result_message=""
    #         print(e)
    #     return result_message
    #
    # file_path="/disk1/work/liaozm/Multi_enzh_Training/Valid_data/zh.json"
    # file1 = open(file_path, 'r', encoding='utf-8')
    # lines1 = file1.readlines()
    # en=[]
    # zh=[]
    # for i in range(200):
    #     try:
    #         x = lines1[i].strip('\n')
    #         y = json.loads(x)
    #         en.append(y["translation"]["en"])
    #         zh.append(y["translation"]["zh"])
    #     except Exception as e:
    #             print(e)
    # en2=[]
    # output_path1="/disk1/work/liaozm/zh_google.txt"
    # output_path2="/disk1/work/liaozm/en_llam.txt"
    #
    # zh_to_en_translate_Prompt = """
    #             现在我给你一句中文，请帮我翻译为英文。例如，我给你的内容为{"zh": "我们联系了阿丁利学院征求意见。"}，则你返回的结果应为{"en": "Ardingly Collage was approached for comment."}。现如今我给你的内容为{"zh":"content"}，请返回给我结果
    #             """
    #
    # for i in range(len(en)):
    #     Prompt=zh_to_en_translate_Prompt.replace("content", zh[i])
    #     message=Message("user",Prompt)
    #     messages=[message]
    #     result=json.loads(post_llm_http_request(messages=messages))["en"]
    #     en2.append(result)
    #     print(i+1,result)




















    # import pandas as pd
    # path="/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Institution/4_special.xls"
    # path2="/disk1/work/liaozm/Multi_enzh_Training/SpecialWords/Institution/x.json"
    # df = pd.read_excel(path)
    # i=0;
    # for indexs in df.index:
    #     i=i+1
    #     # Sentence.append(df.loc[indexs].values[0])
    #     print(i,df.loc[indexs].values[0])
    #     with open(path2, mode='a', encoding='utf-8') as filename:
    #         filename.write(df.loc[indexs].values[0])
    #         filename.write('\n')  # 换行

    # result=(model_infer(model_path, device_num, Sentence, batch_size=1280, L_src=L_src))
    # x=set()
    # for indexs in df.index:
        # if(df.loc[indexs].values[1]==result[indexs]):
        # print(indexs+2,df.loc[indexs].values[0],df.loc[indexs].values[1],result[indexs])
        # len1=len(x)
        # x.add(df.loc[indexs].values[0])
        # len2=len(x)
        # if((len1+1)==len2):
            # print(indexs + 2, df.loc[indexs].values[0], df.loc[indexs].values[1])
            # print('"'+df.loc[indexs].values[0]+'":"'+df.loc[indexs].values[1]+'",')
    # print(len(x))