# Run command : deepspeed --num_gpus 1 deepspeed_multiple_gpu.py  
# --num_gpus set gpu number

import os
import deepspeed
import torch
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, pipeline
import time

local_rank = int(os.getenv('LOCAL_RANK', '0'))
world_size = int(os.getenv('WORLD_SIZE', '1'))

model = AutoModelForSeq2SeqLM.from_pretrained("/disk1/m2mmodels/1_2B/")
tokenizer = AutoTokenizer.from_pretrained("/disk1/m2mmodels/1_2B/")

generator = pipeline('translation', model=model, tokenizer=tokenizer, src_lang="th", tgt_lang='zh', device=local_rank, max_length=400)

generator.model = deepspeed.init_inference(generator.model,
                                           mp_size=2,
                                           dtype=torch.half,
                                           replace_method='auto',
					   replace_with_kernel_inject=True)

for count in range(1, 100):
    time1 = time.time()
    string = generator("รัสเซียโจมตีทางอากาศในยูเครนครั้งใหญ่ที่สุด มุ่งเป้าหลายเมืองรวมถึงกรุงเคียฟ ปูตินลั่น ทำเพื่อล้างแค้นเหตุระเบิดสะพานไครเมีย พร้อมขู่โจมตีเพิ่มอีก")
    time2 = time.time()

    if not torch.distributed.is_initialized() or torch.distributed.get_rank() == 0:
        print(f"Translate Resule [{time2 - time1} s]: {string} ")
