from transformers import M2M100Tokenizer, M2M100ForConditionalGeneration, AutoModelForSeq2SeqLM, AutoTokenizer, pipeline
import time
import torch
import os
from accelerate import Accelerator, notebook_launcher
from torch.utils.data import DataLoader
from datasets import Dataset
import torch.optim as optim

"""
    M2M100 is a multilingual encoder-decoder (seq-to-seq) model primarily intended for translation tasks.
    So there is no need to fine-tune
"""

def main():
    text_to_translate = [
        "Viața este ca o cutie de ciocolată.", # 生活就像一盒巧克力
        "Vei deveni a patra oară mai flexibil", # 你将成为第四次更灵活
        "Domnul meu, căci ei au rătăcit mulţime de oameni!", # 我的主啊！偶像们确已使许多人迷误
        "În prezent, există doar 8 țări mici care nu au încă o bancă centrală"  # 目前只有8小国家还没有中央银行
    ]
#    text_to_translate = [
#        "Viața este ca o cutie de ciocolată.", # 生活就像一盒巧克力
#        "Viața este ca o cutie de ciocolată.",
#        "Viața este ca o cutie de ciocolată.",
#        "Viața este ca o cutie de ciocolată."
#    ]

    data ={"text": text_to_translate}


    google_translate_cn = "生活就像一盒巧克力。"

    model_name = "/disk1/m2mmodels/1_2B/"
    accelerator = Accelerator()

    model = M2M100ForConditionalGeneration.from_pretrained(model_name)

    tokenizer = M2M100Tokenizer.from_pretrained(model_name)
    tokenizer.src_lang = "ro"

    dataset = Dataset.from_dict(data)
    dataloader = DataLoader(dataset, batch_size=1)

    optimizer = torch.optim.Adam(model.parameters())

#    optimizer = optim.Adadelta(model.parameters(), lr=1.0)

    model, optimizer, dataloader = accelerator.prepare(model, optimizer, dataloader)

    model.eval()

    for count in range(1, 100):
        time1 = time.time()
        for batch in dataloader:
            model_inputs = tokenizer(batch["text"], return_tensors="pt", padding=True, max_length=30).to(accelerator.device)
            generated_tokens = accelerator.unwrap_model(model).generate(**model_inputs, forced_bos_token_id=tokenizer.get_lang_id("zh"), max_length=512)
            generated_tokens = accelerator.gather(generated_tokens)
            # res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
            # time2 = time.time()
            # print(f"{count} translate result [{time2 - time1} s]: {res}")

        res = tokenizer.batch_decode(generated_tokens, skip_special_tokens=True)
        time2 = time.time()
        if not torch.distributed.is_initialized() or torch.distributed.get_rank() == 0:
            print(f"{count} translate result [{time2 - time1} s]: {res}")
        time.sleep(5)

if __name__ == "__main__":
    main()
#    notebook_launcher(main, num_processes=1, use_port="29501")

