import json
import logging
import time
import sacrebleu
import ctranslate2
import transformers


log_file_path = 'work/logs/MtModelTraning/evaluation.log'
logging.basicConfig(filename=log_file_path, format='%(asctime)s - %(levelname)s - %(message)s', level=logging.INFO,
                    filemode='a')
logger = logging.getLogger()


def jakozh_length(text):
    if text is None:
        return 0
    lenText = len(text)
    lenText_utf8 = len(text.encode('utf-8'))
    # utf-8一个汉字占3个字符，减去原计数就是多出来的2/3，再除以2就是增量。再加回去即可
    size = int((lenText_utf8) / 3)
    return size


def get_data(data_path, size, max_seq_len, L_src, L_obj):
    datalist = []
    with open(data_path, encoding='utf-8') as file:
        lines = file.readlines()
        i=0
        datalist = [(json.loads(line)['translation'][L_src], json.loads(line)['translation'][L_obj]) for line in lines]
    datalist = datalist[:min(size, len(datalist))]
    src_list = [line[0][:min(max_seq_len, len(line[0]))] for line in datalist]
    tgt_list = [[line[1][:min(max_seq_len, len(line[1]))] for line in datalist]]
    return src_list, tgt_list


def get_model(model_path, device_type, device_num):
    translator = ctranslate2.Translator(model_path, device_type, device_index=device_num)
    tokenizer = transformers.AutoTokenizer.from_pretrained(model_path)
    return translator, tokenizer


def model_infer(translator, tokenizer, valid_data, batch_size=8,L_obj="zh"):

    start_ts_time = time.time()
    sentences = [tokenizer.convert_ids_to_tokens(tokenizer.encode(seq)) for seq in valid_data]
    # target_prefix = [tokenizer.lang_code_to_token[L_obj]]
    target_prefix = ["zho_Hans"]
    results = translator.translate_batch(sentences, target_prefix=[target_prefix] * len(valid_data),max_batch_size=batch_size)


    targets = [result.hypotheses[0][1:] for result in results]
    infer_results = [tokenizer.decode(tokenizer.convert_tokens_to_ids(target)) for target in targets]
    end_ts_time = time.time()


    return infer_results, end_ts_time - start_ts_time


def evaluation(data_path, translator, tokenizer, size, l_src, l_obj, batch_size=8):
    if l_src == "jpn_Jpan" or l_src=="zho_Hans":
        max_seq_len = 256
    else:
        max_seq_len = 320

    # init test data
    valid_data_src, valid_data_obj = get_data(data_path, size, max_seq_len, l_src, l_obj)

    tokenizer.src_lang = l_src
    infer_results, infer_time = model_infer(translator, tokenizer, valid_data_src, batch_size, l_obj)

    bleu = sacrebleu.corpus_bleu(infer_results, valid_data_obj)
    print(f"The_Bleu({l_src})={bleu.score}")

    token_count = sum(map(lambda x: len(tokenizer(x, return_tensors="pt")["input_ids"][0]), valid_data_src))
    if l_src == "ko" or l_src == "ja" or l_src=="zh":
        word_count = sum(map(lambda x: jakozh_length(x), valid_data_src))
    else:
        word_count = sum(map(lambda x: len(x.split(" ")), valid_data_src))
    logger.info(
        f"""======================CT2 infer======================
            Bleu  Info: bleu_score:{bleu.score},lang:{l_src},batch_size:{batch_size},data_path:{data_path}
            Speed Info: word_count:{word_count},word_speed:{word_count / infer_time} words/second,token_count:{token_count},token_speed:{token_count / infer_time} tokens/second 
            """
    )

def other_eval(model_path,device_num):
    valid_data_map = {
        "uzn_Latn": "/disk1/work/MtCorpus/NLLB200_Corpus/Valid_data/uzn_Latn.json",
        "bel_Cyrl": "/disk1/work/MtCorpus/NLLB200_Corpus/Valid_data/bel_Cyrl.json",
        "kir_Cyrl": "/disk1/work/MtCorpus/NLLB200_Corpus/Valid_data/kir_Cyrl.json",
        "lvs_Latn": "/disk1/work/MtCorpus/NLLB200_Corpus/Valid_data/lvs_Latn.json",
    }
    size = 1000
    batch_size =1280
    # init model
    translator, tokenizer = get_model(model_path, "cuda", device_num)

    # evaluation
    for key in valid_data_map.keys():
        evaluation(data_path=valid_data_map[key], translator=translator, tokenizer=tokenizer,
                   l_src=key, l_obj="zho_Hans", size=size, batch_size=batch_size)

if __name__ == "__main__":
    device_num=6
    L_src="other"
    if(L_src=="other"):
        L_obj = "zho_Hans"
        model_path="/disk1/work/MtModel/NLLB200/NLLB_20240111_be_lv_ky_uz_int8"
        other_eval(model_path,device_num)




















